<?php
/**
 * @package   	OneAll Social Login
 * @copyright 	Copyright 2012 http://www.oneall.com - All rights reserved.
 * @license   	GNU/GPL 2 or later
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * The "GNU General Public License" (GPL) is available at
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 *
 */
if (!defined ('_PS_VERSION_'))
{
	exit;
}

class OneallSocialLogin extends Module
{
	/**
	 * Constructor
	 **/
	public function __construct ()
	{
		$this->name = 'oneallsociallogin';
		$this->tab = 'administration';
		$this->version = '1.1';
		$this->author = 'OneAll LLC';
		$this->need_instance = 1;
		$this->secure_key = Tools::encrypt ($this->name);
		$this->module_key = '2571f9dab09af193a8ca375a09133873';

		parent::__construct ();

		$this->displayName = $this->l ('OneAll Social Login');
		$this->description = $this->l ('Professionally developed and free module that allows your users to register and login to PrestaShop with their Social Network account (Twitter, Facebook, LinkedIn, Google ...)');
		$this->confirmUninstall = $this->l ('Are you sure you want to uninstall Social Login?');

		//This is the first time that the class is used
		if (!Configuration::get ('OASL_FIRST_INSTALL'))
		{
			Configuration::updateValue ('OASL_FIRST_INSTALL', '1');

			//API Credentials
			Configuration::updateValue ('OASL_API_HANDLER', 'curl');
			Configuration::updateValue ('OASL_API_PORT', '443');
			Configuration::updateValue ('OASL_PROVIDERS', 'facebook,twitter,google,linkedin');
			Configuration::updateValue ('OASL_LINK_ACCOUNT_DISABLE', 0);

			// Hooks.
			Configuration::updateValue ('OASL_HOOK_LEFT_DISABLE', 1);
			Configuration::updateValue ('OASL_HOOK_LEFT_TITLE', $this->l ('Connect with:'));
			Configuration::updateValue ('OASL_HOOK_RIGHT_DISABLE', 0);
			Configuration::updateValue ('OASL_HOOK_RIGHT_TITLE', $this->l ('Connect with:'));

			// Data Handling.
			Configuration::updateValue ('OASL_DATA_HANDLING', 'verify');

			// Email Settings.
			Configuration::updateValue ('OASL_EMAIL_CUSTOMER_DISABLE', '0');
			Configuration::updateValue ('OASL_EMAIL_ADMIN_DISABLE', '0');
		}

		if (!Configuration::get ('OASL_API_SUBDOMAIN'))
		{
			$this->warning = $this->l('The API Settings must be filled out in order to use this module correctly.');
		}

		//Required includes
		require_once(dirname (__FILE__) . "/includes/tools.php");
		require_once(dirname (__FILE__) . "/includes/providers.php");
	}

	/**
	 * **************************************************************************
	 * Administration Area
	 * **************************************************************************
	 */

	/**
	 * Display Admin Menu
	 **/
	public function getContent ()
	{
		// Compute the form url.
		$form_url = 'index.php?';
		foreach ($_GET as $key => $value)
		{
			if (strtolower ($key) <> 'submit')
			{
				$form_url .= $key . '=' . $value . '&';
			}
		}
		$form_url = rtrim ($form_url, '&');


		// This is what is being displayed.
		$html = '';

		// The submit button has been clicked.
		if (Tools::isSubmit ('submit'))
		{
			// Read API credentials.
			$api_key = trim (Tools::getValue ('OASL_API_KEY'));
			$api_password = trim (Tools::getValue ('OASL_API_PASSWORD'));
			$api_subdomain = strtolower (trim (Tools::getValue ('OASL_API_SUBDOMAIN')));

			// Check if the full domain has been entered.
			if (preg_match ("/([a-z0-9\-]+)\.api\.oneall\.com/i", $api_subdomain, $matches))
			{
				$api_subdomain = $matches [1];
			}

			//Read API Connection Settings
			$api_handler = Tools::getValue ('OASL_API_HANDLER');
			$api_handler = ($api_handler == 'fsockopen' ? 'fsockopen' : 'curl');
			$api_port = Tools::getValue ('OASL_API_PORT');
			$api_port = ($api_port == 80 ? 80 : 443);

			//Read Providers
			$use_provider_keys = array ();
			$tmp_provider_keys = Tools::getValue ('OASL_PROVIDERS');
			if (is_array ($tmp_provider_keys) AND count ($tmp_provider_keys) > 0)
			{
				foreach ($tmp_provider_keys AS $tmp_provider_key)
				{
					$tmp_provider_key = trim (strtolower ($tmp_provider_key));
					if (oneall_social_login_providers::is_valid_key ($tmp_provider_key))
					{
						$use_provider_keys [] = $tmp_provider_key;
					}
				}
			}

			//Hook Left
			$hook_left_disable = (Tools::getValue ('OASL_HOOK_LEFT_DISABLE') == 1 ? 1 : 0);
			$hook_left_title = trim (Tools::getValue ('OASL_HOOK_LEFT_TITLE'));

			//Hook Right
			$hook_right_disable = (Tools::getValue ('OASL_HOOK_RIGHT_DISABLE') == 1 ? 1 : 0);
			$hook_right_title = trim (Tools::getValue ('OASL_HOOK_RIGHT_TITLE'));

			//Settings
			$link_account_disable = (Tools::getValue ('OASL_LINK_ACCOUNT_DISABLE') == 1 ? 1 : 0);
			$data_handling = Tools::getValue ('OASL_DATA_HANDLING');
			$data_handling = (in_array ($data_handling, array ('ask', 'verify', 'auto')) ? $data_handling : 'verify');
			$email_customer_disable = (Tools::getValue ('OASL_EMAIL_CUSTOMER_DISABLE') == 1 ? 1 : 0);
			$email_admin_disable = (Tools::getValue ('OASL_EMAIL_ADMIN_DISABLE') == 1 ? 1 : 0);

			//Save Values
			Configuration::updateValue ('OASL_API_KEY', $api_key);
			Configuration::updateValue ('OASL_API_PASSWORD', $api_password);
			Configuration::updateValue ('OASL_API_SUBDOMAIN', $api_subdomain);
			Configuration::updateValue ('OASL_API_HANDLER', $api_handler);
			Configuration::updateValue ('OASL_API_PORT', $api_port);
			Configuration::updateValue ('OASL_PROVIDERS', implode (',', $use_provider_keys));
			Configuration::updateValue ('OASL_HOOK_LEFT_DISABLE', $hook_left_disable);
			Configuration::updateValue ('OASL_HOOK_LEFT_TITLE', $hook_left_title);
			Configuration::updateValue ('OASL_HOOK_RIGHT_DISABLE', $hook_right_disable);
			Configuration::updateValue ('OASL_HOOK_RIGHT_TITLE', $hook_right_title);
			Configuration::updateValue ('OASL_LINK_ACCOUNT_DISABLE', $link_account_disable);
			Configuration::updateValue ('OASL_DATA_HANDLING', $data_handling);
			Configuration::updateValue ('OASL_EMAIL_ADMIN_DISABLE', $email_admin_disable);
			Configuration::updateValue ('OASL_EMAIL_CUSTOMER_DISABLE', $email_customer_disable);
		}

		//Read API Credentials
		$api_key = Configuration::get ('OASL_API_KEY');
		$api_password = Configuration::get ('OASL_API_PASSWORD');
		$api_subdomain = Configuration::get ('OASL_API_SUBDOMAIN');

		//Read API Connection Settings
		$api_handler = Configuration::get ('OASL_API_HANDLER');
		$api_handler = ($api_handler == 'fsockopen' ? 'fsockopen' : 'curl');
		$api_port = Configuration::get ('OASL_API_PORT');
		$api_port = ($api_port == 80 ? 80 : 443);

		//Hook Left
		$hook_left_disable = Configuration::get ('OASL_HOOK_LEFT_DISABLE') == 1 ? 1 : 0;
		$hook_left_title = Configuration::get ('OASL_HOOK_LEFT_TITLE');

		//Hook Right
		$hook_right_disable = Configuration::get ('OASL_HOOK_RIGHT_DISABLE') == 1 ? 1 : 0;
		$hook_right_title = Configuration::get ('OASL_HOOK_RIGHT_TITLE');

		//Settings
		$link_account_disable = Configuration::get ('OASL_LINK_ACCOUNT_DISABLE') == 1 ? 1 : 0;
		$data_handling = Configuration::get ('OASL_DATA_HANDLING');
		$data_handling = (in_array ($data_handling, array ('ask', 'verify', 'auto')) ? $data_handling : 'verify');
		$email_customer_disable = Configuration::get ('OASL_EMAIL_CUSTOMER_DISABLE') == 1 ? 1 : 0;
		$email_admin_disable = Configuration::get ('OASL_EMAIL_ADMIN_DISABLE') == 1 ? 1 : 0;

		//Read providers
		$use_provider_keys = array ();
		$tmp_provider_keys = explode (',', Configuration::get ('OASL_PROVIDERS'));
		if (is_array ($tmp_provider_keys) AND count ($tmp_provider_keys) > 0)
		{
			foreach ($tmp_provider_keys AS $tmp_provider_key)
			{
				if (oneall_social_login_providers::is_valid_key ($tmp_provider_key))
				{
					$use_provider_keys [] = $tmp_provider_key;
				}
			}
		}

		$html .= '
			<script type="text/javascript" src="' . (__PS_BASE_URI__) . 'modules/' . $this->name . '/assets/js/admin.js"></script>
			<link type="text/css" href="' . (__PS_BASE_URI__) . 'modules/' . $this->name . '/assets/css/admin.css" rel="stylesheet"></link>
			<h2>' . $this->l ('OneAll Social Login') . '</h2>
			<p>
				' . $this->l ('Allow your visitors to comment, login and register with 20+ Social Networks like for example Twitter, Facebook, LinkedIn, Hyves, VKontakte, Google or Yahoo.') . '
				' . $this->l ('Draw a larger audience and increase your user engagement and conversion rates in a few simple steps.') . '
			</p>
		<div class="oasl_box">
			<div class="oasl_box_title">' . $this->l ('The setup takes only a couple of minutes!') . '</div>
				<p>
					' . $this->l ('To be able to use this plugin you first of all need to create a free account at') . ' <a href="http://app.oneall.com" target="_blank">http://www.oneall.com</a> ' . $this->l ('and setup a Site.') . '
				</p>
				<p>
					' . $this->l ('After having created your account and setup your Site, please enter the Site settings in the form <strong>API Settings</strong> below.') . '
				</p>
				<p>
					<a href="http://app.oneall.com" target="_blank" class="oasl_button">' . $this->l ('Click here to setup your free account') . '</a>
				</p>
		</div>
		<form action="' . Tools::safeOutput ($form_url) . '" method="post">
			<fieldset>
				<legend>' . $this->l ('API Connection') . '</legend>
				<label>' . $this->l ('API Connection Handler:') . '</label>
				<div class="margin-form" style="margin-bottom: 20px;">
					<input type="radio" name="OASL_API_HANDLER" id="OASL_API_HANDLER_CURL" value="curl" ' . ($api_handler <> 'fsockopen' ? 'checked="checked"' : '') . ' /> ' . $this->l ('Use PHP CURL to communicate with the API') . ' <strong>(' . $this->l ('Default') . ')</strong>
					<p>' . $this->l ('Using CURL is recommended but it might be disabled on some servers.') . '</p><br />
					<input type="radio" name="OASL_API_HANDLER" id="OASL_API_HANDLER_FSOCKOPEN" value="fsockopen" ' . ($api_handler == 'fsockopen' ? 'checked="checked"' : '') . ' /> ' . $this->l ('Use PHP FSOCKOPEN to communicate with the API') . '
					<p>' . $this->l ('Try using FSOCKOPEN if you encounter any problems with CURL.') . '</p>
				</div>
				<label>' . $this->l ('API Connection Port:') . '</label>
				<div class="margin-form">
					<input type="radio" name="OASL_API_PORT" value="443" id="OASL_API_PORT_443" ' . ($api_port <> 80 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Communication via HTTPS on port 443') . ' <strong>(' . $this->l ('Default') . ')</strong>
					<p>' . $this->l ('Using port 443 is secure but you might need OpenSSL.') . '</p><br />
					<input type="radio" name="OASL_API_PORT" value="80" id="OASL_API_PORT_80" ' . ($api_port == 80 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Communication via HTTP on port 80') . '
					<p>' . $this->l ('Using port 80 is a bit faster, does not need OpenSSL but is also less secure.') . '</p>
				</div>
				<div class="margin-form">
					<input type="button" id="OASL_VERIFY_CONNECTION_HANDLER" value="' . $this->l ('AutoDetect the best API Connection Settings') . '" class="oasl_button" />
				</div>
				<div class="margin-form">
					<span class="oasl_message" id="OASL_VERIFY_CONNECTION_HANDLER_RESULT"></span>
				</div>
			</fieldset>

			<fieldset style="margin-top:20px">
				<div class="oasl_notice">
					<a target="_blank" href="https://app.oneall.com/applications/">' . $this->l ('Click here to create and view your API Credentials') . '</a>
				</div>
				<legend>' . $this->l ('API Settings') . '</legend>
				<label>' . $this->l ('API Subdomain') . ':</label>
				<div class="margin-form">
					<input type="text" name="OASL_API_SUBDOMAIN" id="OASL_API_SUBDOMAIN" size="60" value="' . htmlspecialchars ($api_subdomain) . '" />
				</div>
				<label>' . $this->l ('API Public Key') . ':</label>
				<div class="margin-form">
					<input type="text" name="OASL_API_KEY" id="OASL_API_KEY" size="60" value="' . htmlspecialchars ($api_key) . '" />
				</div>
				<label>' . $this->l ('API Private Key') . ':</label>
				<div class="margin-form">
					<input type="text" name="OASL_API_PASSWORD" id="OASL_API_PASSWORD" size="60" value="' . htmlspecialchars ($api_password) . '" />
				</div>
				<div class="margin-form">
					<input type="button" id="OASL_VERIFY_CONNECTION_SETTINGS" value="' . $this->l ('Verify the API Settings') . '" class="oasl_button" />
				</div>
				<div class="margin-form">
					<span class="oasl_message" id="OASL_VERIFY_CONNECTION_SETTINGS_RESULT"></span>
				</div>
			</fieldset>

			<fieldset style="margin-top:20px">
				<legend>' . $this->l ('Hook: Left Side') . '</legend>
				<div class="oasl_notice">' . $this->l ('Displays Social Login on the left side of your shop') . '</div>
				<label>' . $this->l ('Enable Left Side Hook?') . '</label>
				<div class="margin-form" style="margin-bottom: 20px;">
					<input type="radio" name="OASL_HOOK_LEFT_DISABLE" id="OASL_HOOK_LEFT_DISABLE_0" value="0" ' . ($hook_left_disable <> 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Enable') . '&nbsp;
					<input type="radio" name="OASL_HOOK_LEFT_DISABLE" id="OASL_HOOK_LEFT_DISABLE_1" value="1" ' . ($hook_left_disable == 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Disable') . '<br />
				</div>
				<label>' . $this->l ('Left Side Caption') . ':</label>
				<div class="margin-form">
					<input type="text" name="OASL_HOOK_LEFT_TITLE" id="OASL_HOOK_LEFT_TITLE" size="60" value="' . htmlspecialchars ($hook_left_title) . '" />
					<p>	' . $this->l ('Leave empty for none.') . '</p>
				</div>
			</fieldset>

			<fieldset style="margin-top:20px">
				<legend>' . $this->l ('Hook: Right Side') . '</legend>
				<div class="oasl_notice">' . $this->l ('Displays Social Login on the right side of your shop') . '</div>
				<label>' . $this->l ('Enable Right Side Hook?') . '</label>
				<div class="margin-form" style="margin-bottom: 20px;">
					<input type="radio" name="OASL_HOOK_RIGHT_DISABLE" id="OASL_HOOK_RIGHT_DISABLE_0" value="0" ' . ($hook_right_disable <> 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Enable') . '&nbsp;
					<input type="radio" name="OASL_HOOK_RIGHT_DISABLE" id="OASL_HOOK_RIGHT_DISABLE_1" value="1" ' . ($hook_right_disable == 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Disable') . '<br />
				</div>
				<label>' . $this->l ('Right Side Caption') . ':</label>
				<div class="margin-form">
					<input type="text" name="OASL_HOOK_RIGHT_TITLE" id="OASL_HOOK_RIGHT_TITLE" size="60" value="' . htmlspecialchars ($hook_right_title) . '" />
					<p>' . $this->l ('Leave empty for none.') . '</p>
				</div>
			</fieldset>

			<fieldset style="margin-top:20px">
				<legend>' . $this->l ('Settings') . '</legend>

				<label>' . $this->l ('Enable Administrator Emails?') . '</label>
				<div class="margin-form" style="margin-bottom: 20px;">
					<input type="radio" name="OASL_EMAIL_ADMIN_DISABLE" id="OASL_EMAIL_ADMIN_DISABLE_0" value="0" ' . ($email_admin_disable <> 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Enable') . ' <strong>(' . $this->l ('Default') . ')</strong>
					<p>' . $this->l ("Tick to have the module send an email to the administrators for each customer that registers with Social Login") . '</p><br />
					<input type="radio" name="OASL_EMAIL_ADMIN_DISABLE" id="OASL_EMAIL_ADMIN_DISABLE_1" value="1" ' . ($email_admin_disable == 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Disable') . '
					<p>' . $this->l ('Tick to disable the emails send to administrators.') . '</p>
				</div>

				<label>' . $this->l ('Enable Customer Emails?') . '</label>
				<div class="margin-form" style="margin-bottom: 20px;">
					<input type="radio" name="OASL_EMAIL_CUSTOMER_DISABLE" id="OASL_EMAIL_CUSTOMER_DISABLE_0" value="0" ' . ($email_customer_disable <> 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Enable') . ' <strong>(' . $this->l ('Default') . ')</strong>
					<p>' . $this->l ("Tick to have the module send an email to each new customer that registers with Social Login") . '</p><br />
					<input type="radio" name="OASL_EMAIL_CUSTOMER_DISABLE" id="OASL_EMAIL_CUSTOMER_DISABLE_0" value="1" ' . ($email_customer_disable == 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Disable') . '
					<p>' . $this->l ('Tick to disable the emails send to customer.') . '</p>
				</div>


				<label>' . $this->l ('Enable Account Linking?') . '</label>
				<div class="margin-form" style="margin-bottom: 20px;">
					<input type="radio" name="OASL_LINK_ACCOUNT_DISABLE" id="OASL_LINK_ACCOUNT_DISABLE_0" value="0" ' . ($link_account_disable <> 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Enable Account Linking') . ' <strong>(' . $this->l ('Default') . ')</strong>
					<p>' . $this->l ("If the user's social network profile provides a verified email address, the plugin will try to link the profile to an existing account.") . '</p><br />
					<input type="radio" name="OASL_LINK_ACCOUNT_DISABLE" id="OASL_LINK_ACCOUNT_DISABLE_1" value="1" ' . ($link_account_disable == 1 ? 'checked="checked"' : '') . ' /> ' . $this->l ('Disable Account Linking') . '
					<p>' . $this->l ('Social network profiles will never be linked automatically to existing users.') . '</p>
				</div>

				<label>' . $this->l ('User Data Completion?') . '</label>
				<div class="margin-form" style="margin-bottom: 20px;">
					<p>' . $this->l ('To create an account PrestaShop requires a firstname, a lastname and an email address. Some social networks (i.e. Twitter) however do not provide this data.') . '</p><br />
					<input type="radio" name="OASL_DATA_HANDLING" id="OASL_DATA_HANDLING_VERIFY" value="verify" ' . (!in_array ($data_handling, array (
				'auto, ask'
			)) ? 'checked="checked"' : '') . ' /> ' . $this->l ('Always ask users to verify their data when they sign up with a social network') . ' <strong>(' . $this->l ('Default') . ')</strong>
					<p>' . $this->l ('Tick this option to have the users always verify the data retrieved from their social network profiles.') . '</p><br />
					<input type="radio" name="OASL_DATA_HANDLING" id="OASL_DATA_HANDLING_ASK" value="ask" ' . ($data_handling == 'ask' ? 'checked="checked"' : '') . ' /> ' . $this->l ('Only ask for missing values') . ' (' . $this->l ('Faster Registration') . ')
					<p>' . $this->l ('Tick this option to have the users verify their data manually only in case there are required fields that are not provided by the social network.') . '</p><br />
					<input type="radio" name="OASL_DATA_HANDLING" id="OASL_DATA_HANDLING_AUTO" value="auto" ' . ($data_handling == 'auto' ? 'checked="checked"' : '') . ' /> ' . $this->l ('Never ask, create placeholders for missing fields') . ' (' . $this->l ('Fastests Registration, Not recommended') . ')
					<p>' . $this->l ('Tick this option to have the plugin automatically create placeholder values for fields that are not provided by the social network.') . '</p>
				</div>
			</fieldset>


			<fieldset style="margin-top:20px">
					<legend>' . $this->l ('Tick the social networks to enable') . '</legend>';

		//Add providers
		$providers = oneall_social_login_providers::get_list ();
		foreach ($providers AS $key => $data)
		{
			$provider_key = 'OASL_PROVIDER_' . strtoupper ($key);

			$html .= '<div class="oasl_provider_row">
									<label for="' . $provider_key . '">' . $data ['name'] . '</label>
										<div class="margin-form">
											<div class="oasl_provider_block">
												<span class="oasl_provider oasl_provider_' . $key . '" data-for="' . $provider_key . '"></span>
												<input type="checkbox" id="' . $provider_key . '" name="OASL_PROVIDERS[]" value="' . $key . '" ' . (in_array ($key, $use_provider_keys) ? 'checked="checked"' : '') . ' />
											</div>
									</div>
								</div>';
		}
		$html .= '</fieldset>';

		$html .= '
			<fieldset style="margin-top:20px">
				<legend>' . $this->l ('Save API Settings') . '</legend>
				<div class="margin-form">
					<input type="submit" class="oasl_button" name="submit" value="' . $this->l ('Save Social Login Settings') . '">
				</div>
			</fieldset>
			<script type="text/javascript">
				<!--
					var OASL_AJAX_TOKEN = \'' . sha1 (_COOKIE_KEY_ . 'ONEALLSOCIALLOGIN') . '\';
					var OASL_AJAX_PATH  = \'' . Tools::getShopDomain (true, true) . __PS_BASE_URI__ . 'modules/oneallsociallogin/assets/ajax/admin.php\';
				//-->
			</script>
		</form>';

		return $html;

	}

	/**
	 * **************************************************************************
	 * INSTALLATION
	 * **************************************************************************
	 */

	protected static function display_error ($error)
	{
		echo '<div class="error"><h3>OneAll Social Login Error:</h3><ol><li>' . $error . '</li></ol></div>';
	}

	/**
	 * Moves a hook to the given position
	 **/
	protected function move_hook_position ($hook_name, $position)
	{
		// Get the hook identifier.
		if (($id_hook = Hook::get ($hook_name)) !== false)
		{
			// Load the module.
			if (($module = Module::getInstanceByName ($this->name)) !== false)
			{
				// Get the max position of this hook.
				$sql = "SELECT MAX(position) AS position FROM `" . _DB_PREFIX_ . "hook_module` WHERE `id_hook` = '" . intval ($id_hook) . "'";
				$result = Db::getInstance ()->GetRow ($sql);
				if (is_array ($result) AND isset ($result ['position']))
				{
					$way = (($result ['position'] >= $position) ? 0 : 1);
					return $module->updatePosition ($id_hook, $way, $position);
				}
			}
		}

		//An error occurred.
		return false;
	}


	/**
	 * Returns a list of files to install
	 **/
	protected function get_files_to_install ()
	{
		return array (
			'oneallsociallogin.php' => array (
				'source' => _PS_MODULE_DIR_ . $this->name . '/upload/',
				'target' => _PS_ROOT_DIR_ . '/'
			),
			'OneAllSocialLoginController.php' => array (
				'source' => _PS_MODULE_DIR_ . $this->name . '/upload/controllers/',
				'target' => _PS_ROOT_DIR_ . '/controllers/'
			),
			'oneallsociallogin.tpl' => array (
				'source' => _PS_MODULE_DIR_ . $this->name . '/upload/themes/default/',
				'target' => _PS_THEME_DIR_
			),
			'oneallsociallogin.css' => array (
				'source' => _PS_MODULE_DIR_ . $this->name . '/upload/themes/default/css/',
				'target' => _PS_THEME_DIR_ . 'css/'
			),
			'oneallsociallogin_account.html' => array (
				'source' => _PS_MODULE_DIR_ . $this->name . '/upload/mails/en/',
				'target' => _PS_MAIL_DIR_ . 'en/'
			),
			'oneallsociallogin_account.txt' => array (
				'source' => _PS_MODULE_DIR_ . $this->name . '/upload/mails/en/',
				'target' => _PS_MAIL_DIR_ . 'en/'
			)
		);
	}

	/**
	 * Returns a list of hooks to install
	 **/
	protected function get_hooks_to_install ()
	{
		return array (
			// Widget
			'leftColumn' => array (
				'pos' => 1
			),
			// Widget
			'rightColumn' => array (
				'pos' => 1
			),
			// Callback
			'top' => array (),
			//Library
			'header' => array ()
		);
	}


	/**
	 * Install
	 **/
	public function install ()
	{
		// Check PS version compliancy (Needs to be lower or equal to 1.4.9)
		if ( ! version_compare(_PS_VERSION_, '1.4.9', '<='))
		{
			self::display_error ('This version of Social Login can only be used with PrestaShop 1.4.9 or less, your version of PrestaShop is: '._PS_VERSION_);
			return false;
		}

		// Start Installation
		if (!parent::install ())
		{
			return false;
		}

		//Store the added files
		$files_added = array ();

		// Copy controller files.
		$files = $this->get_files_to_install ();
		foreach ($files AS $file_name => $file_data)
		{
			if (is_array ($file_data) AND !empty ($file_data ['source']) AND !empty ($file_data ['target']))
			{
				if (!file_exists ($file_data ['target'] . $file_name))
				{
					if (!@copy ($file_data ['source'] . $file_name, $file_data ['target'] . $file_name))
					{
						// Add Error
						self::display_error ('Could not copy the file ' . $file_name . ' to the directory ' . $file_data ['target']);

						// Remove the copied files.
						foreach ($files_added AS $file_name)
						{
							if (file_exists ($file_name))
							{
								@unlink ($file_name);
							}
						}

						// Abort Installation.
						return false;
					}
					else
					{
						$files_added [] = $file_data ['target'] . $file_name;
					}
				}
			}
		}

		// Install our hooks.
		$hooks = $this->get_hooks_to_install ();
		foreach ($hooks AS $hook_name => $hook_data)
		{
			if (!$this->registerHook ($hook_name))
			{
				self::display_error ('Could not register the hook ' . $hook_name);
				return false;
			}
			else
			{
				if (is_array ($hook_data) AND isset ($hook_data ['pos']))
				{
					$this->move_hook_position ($hook_name, $hook_data ['pos']);
				}
			}
		}

		// Create user_token table.
		$query = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'oasl_user` (`id_oasl_user` int(10) unsigned NOT NULL AUTO_INCREMENT, `id_customer` int(10) unsigned NOT NULL, `user_token` varchar(48) NOT NULL, `date_add` datetime NOT NULL, PRIMARY KEY (`id_oasl_user`))';
		if (!Db::getInstance ()->execute ($query))
		{
			self::display_error ("Could not create the table " . _DB_PREFIX_ . "oasl_user");
			return false;
		}

		// Create identity_token table.
		$query = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'oasl_identity` (`id_oasl_identity` int(10) unsigned NOT NULL AUTO_INCREMENT, `id_oasl_user` int(10) unsigned NOT NULL, `identity_token` varchar(48) NOT NULL, `identity_provider` varchar(64) NOT NULL, `num_logins` int(10) unsigned NOT NULL, `date_add` datetime NOT NULL, `date_upd` datetime NOT NULL, PRIMARY KEY (`id_oasl_identity`))';
		if (!Db::getInstance ()->execute ($query))
		{
			self::display_error ("Could not create the table " . _DB_PREFIX_ . "oasl_identity");
			return false;
		}

		//Done
		return true;
	}


	/**
	 * Uninstall
	 **/
	public function uninstall ()
	{
		//UnInstall
		if (!parent::uninstall ())
		{
			return false;
		}

		//Drop user_token table
		$query = 'DROP table IF EXISTS `' . _DB_PREFIX_ . 'oasl_user`';
		Db::getInstance ()->execute ($query);

		//Drop identity_token table
		$query = 'DROP table IF EXISTS `' . _DB_PREFIX_ . 'oasl_identity`';
		Db::getInstance ()->execute ($query);

		// Remove controller files.
		$files = $this->get_files_to_install ();
		foreach ($files AS $file_name => $file_data)
		{
			if (is_array ($file_data) AND !empty ($file_data ['source']) AND !empty ($file_data ['target']))
			{
				if (file_exists ($file_data ['target'] . $file_name))
				{
					@unlink ($file_data ['target'] . $file_name);
				}
			}
		}
	}


	/**
	 * **************************************************************************
	 * HOOKS
	 * **************************************************************************
	 */


	/**
	 * Generic Hook
	 **/
	protected function hookGeneric ($params, $target)
	{
		global $smarty, $cookie;

		//Do not display for users that are logged in
		if (!$cookie->isLogged ())
		{
			//Default
			$widget_enable = false;
			$widget_location = 'unspecified';
			$widget_title = '';

			//Check what has to be done
			switch ($target)
			{
				//Left Column
				case 'left_column':
					if (Configuration::get ('OASL_HOOK_LEFT_DISABLE') <> 1)
					{
						$widget_enable = true;
						$widget_location = 'left';
						$widget_title = Configuration::get ('OASL_HOOK_LEFT_TITLE');
					}
					break;

				//Right Column
				case 'right_column':
					if (Configuration::get ('OASL_HOOK_RIGHT_DISABLE') <> 1)
					{
						$widget_enable = true;
						$widget_location = 'right';
						$widget_title = Configuration::get ('OASL_HOOK_RIGHT_TITLE');
					}
					break;

				//Right Column
				case 'custom':
					$widget_enable = true;
					$widget_location = 'custom';
					$widget_title = '';
					break;
			}

			//	Enable this widget?
			if ($widget_enable)
			{
				//Read Settings
				$providers = explode (',', trim (Configuration::get ('OASL_PROVIDERS')));
				if (is_array ($providers) AND count ($providers) > 0)
				{
					//Setup placeholders
					$smarty->assign ('oasl_widget_title', $widget_title);
					$smarty->assign ('oasl_widget_location', $widget_location);
					$smarty->assign ('oasl_widget_rnd', mt_rand (99999, 9999999));
					$smarty->assign ('oasl_widget_css', '');
					$smarty->assign ('oasl_widget_providers', implode ('","', $providers));

					//Display template
					return $this->display (__FILE__, 'oneallsociallogin.tpl');
				}
			}
		}
	}

	/**
	 * Hook: Left Column
	 **/
	public function hookLeftColumn ($params)
	{
		return $this->hookGeneric ($params, 'left_column');
	}


	/**
	 * Hook: Right Column
	 **/
	public function hookRightColumn ($params)
	{
		return $this->hookGeneric ($params, 'right_column');
	}

	/**
	 * Hook: Header (Library)
	 **/
	public function hookHeader ($params)
	{
		//Output
		$output = '';

		//Read API Credentials
		$api_subdomain = Configuration::get ('OASL_API_SUBDOMAIN');

		//Subdomain is required
		if (!empty ($api_subdomain))
		{
			global $smarty;

			// Add a shortcut.
			$smarty->assign ('HOOK_OASL_CUSTOM', $this->hookGeneric ($params, 'custom'));

			// Add the OneAll Social Library.
			$smarty->assign ('oasl_widget_location', 'library');
			$smarty->assign ('oasl_subdomain', $api_subdomain);
			$output .= $this->display (__FILE__, 'oneallsociallogin.tpl');
		}
		return $output;
	}


	/**
	 * Hook: Page Top (Callback)
	 **/
	public function hookTop ()
	{
		global $cookie;

		// Only of the user is not logged in.
		if (!$cookie->isLogged ())
		{
			// Check for callback arguments.
			if (Tools::getIsset ('oa_action') === true AND Tools::getIsset ('connection_token') === true)
			{
				// Extract the callback arguments.
				$oa_action = trim (Tools::getValue ('oa_action'));
				$connection_token = trim (Tools::getValue ('connection_token'));

				//Verify arguments
				if ($oa_action == 'social_login' AND strlen ($connection_token) > 0)
				{
					// Read the API credentials.
					$api_key = Configuration::get ('OASL_API_KEY');
					$api_password = Configuration::get ('OASL_API_PASSWORD');
					$api_subdomain = Configuration::get ('OASL_API_SUBDOMAIN');

					// Read the API settings.
					$api_handler = Configuration::get ('OASL_API_HANDLER');
					$api_handler = ($api_handler == 'fsockopen' ? 'fsockopen' : 'curl');
					$api_port = Configuration::get ('OASL_API_PORT');
					$api_port = ($api_port == 80 ? 80 : 443);

					// Set API resource uri.
					$api_resource = (($api_port === 443) ? 'https' : 'http') . '://' . $api_subdomain . '.api.oneall.com/connections/' . $connection_token . '.json';

					// Setup API parameters.
					$api_params = array ();
					$api_params ['api_key'] = $api_key;
					$api_params ['api_secret'] = $api_password;

					// Retrieve connection details.
					$result = oneall_social_login_tools::do_api_request ($api_handler, $api_resource, $api_params, 15);

					// Extract the social data.
					if (is_array (($data = oneall_social_login_tools::extract_social_network_profile ($result))))
					{
						// Get the customer identifier for a given token.
						$id_customer_tmp = oneall_social_login_tools::get_id_customer_for_user_token ($data ['user_token']);

						// This customer already exists.
						if (is_numeric ($id_customer_tmp))
						{
							// Update the identity.
							oneall_social_login_tools::update_identity_logins ($data ['identity_token']);

							// Login this customer.
							$id_customer = $id_customer_tmp;
						}
						// This is a new customer.
						else
						{
							// Account linking is enabled.
							if (Configuration::get ('OASL_LINK_ACCOUNT_DISABLE') <> 1)
							{
								// Account linking only works if the email address has been verified.
								if (!empty ($data ['user_email']) && $data ['user_email_is_verified'] === true)
								{
									// Try to read the existing customer account.
									if (($id_customer_tmp = oneall_social_login_tools::get_id_customer_for_email_address ($data ['user_email'])) !== false)
									{
										// Tie the user_token to the customer.
										if (oneall_social_login_tools::link_tokens_to_id_customer ($id_customer_tmp, $data ['user_token'], $data ['identity_token'], $data ['identity_provider']) === true)
										{
											// Update the identity.
											oneall_social_login_tools::update_identity_logins ($data ['identity_token']);

											// Login this customer.
											$id_customer = $id_customer_tmp;
										}
									}
								}
							}
						}

						// Create a user new account.
						if (empty ($id_customer))
						{
							// Notify the customer ?
							$customer_email_notify = true;

							// Redirection url.
							$redirect_to = rtrim (Tools::getShopDomainSsl (true), ' /') . '/oneallsociallogin.php?back=' . urlencode (oneall_social_login_tools::get_current_url ());

							// How do we have to proceed?
							switch (Configuration::get ('OASL_DATA_HANDLING'))
							{
								// Automatic Completion.
								case 'auto':
								// Generate a random email none is provided or if it's already taken.
									if (empty ($data ['user_email']) OR oneall_social_login_tools::get_id_customer_for_email_address ($data ['user_email']) !== false)
									{
										// Generate a random email.
										$data ['user_email'] = oneall_social_login_tools::generate_random_email_address ();

										// But do not send notifications to this email
										$customer_email_notify = false;
									}

									// Generate a lastname if none is provided.
									if (empty ($data ['user_last_name']))
									{
										$data ['user_last_name'] = 'Doe';
									}

									// Generate a firstname if none is provided.
									if (empty ($data ['user_first_name']))
									{
										$data ['user_first_name'] = 'John';
									}
									break;

								//Ask for manual completion if any of the fields is empty or if the email is already taken.
								case 'ask':
									if (empty ($data ['user_email']) OR empty ($data ['user_first_name']) OR empty ($data ['user_last_name']) OR oneall_social_login_tools::get_id_customer_for_email_address ($data ['user_email']) !== false)
									{
										// Save the data in the session.
										$cookie->oasl_data = @serialize ($data);

										//Redirect to the request form
										Tools::redirectLink ($redirect_to);
									}
									break;

								//Always verify the fields
								default:
									// Save the data in the session.
									$cookie->oasl_data = @serialize ($data);

									//Redirect to the request form
									Tools::redirectLink ($redirect_to);
									break;
							}

							// Email flags.
							$send_email_to_admin = ((Configuration::get ('OASL_EMAIL_ADMIN_DISABLE') <> 1) ? true : false);
							$send_email_to_customer = ($customer_email_notify == true AND Configuration::get ('OASL_EMAIL_CUSTOMER_DISABLE') <> 1);

							// Create a new account.
							$id_customer = oneall_social_login_tools::create_customer_from_data ($data, $send_email_to_admin, $send_email_to_customer);
						}

						// Login.
						if (!empty ($id_customer) AND oneall_social_login_tools::login_customer ($id_customer))
						{
							//Remove the data (Should not be set here)
							if (isset ($cookie->oasl_data))
							{
								unset ($cookie->oasl_data);
							}

							//A refresh is required to update the page
							$back = trim (Tools::getValue ('back'));
							$back = (!empty ($back) ? $back : oneall_social_login_tools::get_current_url ());
							Tools::redirectLink ($back);
						}
					}
				}
			}
		}
	}
}
